"""Endpoint catalog starter for multi-version wrappers.

This is intentionally small in v0.1 and meant to be expanded/generated.
"""

ENDPOINT_CATALOG: dict[str, dict[str, dict[str, str]]] = {
    "workspace": {
        "2.0": {
            "clusters_create": "/api/2.0/clusters/create",
            "clusters_delete": "/api/2.0/clusters/delete",
            "clusters_edit": "/api/2.0/clusters/edit",
            "clusters_events": "/api/2.0/clusters/events",
            "clusters_get": "/api/2.0/clusters/get",
            "clusters_list": "/api/2.0/clusters/list",
            "clusters_permanent_delete": "/api/2.0/clusters/permanent-delete",
            "clusters_restart": "/api/2.0/clusters/restart",
            "clusters_start": "/api/2.0/clusters/start",
            "repos_update": "/api/2.0/repos/{repo_id}",
            "repos_list": "/api/2.0/repos",
            "repos_get": "/api/2.0/repos/{repo_id}",
            "repos_create": "/api/2.0/repos",
            "repos_delete": "/api/2.0/repos/{repo_id}",
            "secrets_put": "/api/2.0/secrets/put",
            "secret_scopes_create": "/api/2.0/secrets/scopes/create",
            "secret_scopes_list": "/api/2.0/secrets/scopes/list",
            "secret_scopes_delete": "/api/2.0/secrets/scopes/delete",
            "token_create": "/api/2.0/token/create",
            "token_delete": "/api/2.0/token/delete",
            "token_list": "/api/2.0/token/list",
            "token_revoke": "/api/2.0/token/delete",
            "sql_warehouses_list": "/api/2.0/sql/warehouses",
            "sql_warehouses_get": "/api/2.0/sql/warehouses/{warehouse_id}",
            "sql_warehouses_create": "/api/2.0/sql/warehouses",
            "sql_warehouses_edit": "/api/2.0/sql/warehouses/{warehouse_id}/edit",
            "sql_warehouses_delete": "/api/2.0/sql/warehouses/{warehouse_id}",
            "instance_pools_list": "/api/2.0/instance-pools/list",
            "instance_pools_get": "/api/2.0/instance-pools/get",
            "instance_pools_create": "/api/2.0/instance-pools/create",
            "instance_pools_edit": "/api/2.0/instance-pools/edit",
            "instance_pools_delete": "/api/2.0/instance-pools/delete",
            "cluster_policies_list": "/api/2.0/policies/clusters/list",
            "cluster_policies_get": "/api/2.0/policies/clusters/get",
            "cluster_policies_create": "/api/2.0/policies/clusters/create",
            "cluster_policies_edit": "/api/2.0/policies/clusters/edit",
            "cluster_policies_delete": "/api/2.0/policies/clusters/delete",
            "dbfs_list": "/api/2.0/dbfs/list",
            "dbfs_get_status": "/api/2.0/dbfs/get-status",
            "dbfs_read": "/api/2.0/dbfs/read",
            "dbfs_delete": "/api/2.0/dbfs/delete",
            "dbfs_mkdirs": "/api/2.0/dbfs/mkdirs",
            "permissions_jobs_get": "/api/2.0/permissions/jobs/{job_id}",
            "permissions_jobs_patch": "/api/2.0/permissions/jobs/{job_id}",
            "permissions_jobs_permission_levels": "/api/2.0/permissions/jobs/{job_id}/permissionLevels",
            "permissions_clusters_get": "/api/2.0/permissions/clusters/{cluster_id}",
            "permissions_clusters_patch": "/api/2.0/permissions/clusters/{cluster_id}",
            "permissions_clusters_permission_levels": "/api/2.0/permissions/clusters/{cluster_id}/permissionLevels",
            "permissions_repos_get": "/api/2.0/permissions/repos/{repo_id}",
            "permissions_repos_patch": "/api/2.0/permissions/repos/{repo_id}",
            "permissions_repos_permission_levels": "/api/2.0/permissions/repos/{repo_id}/permissionLevels",
        },
        "2.1": {
            "jobs_create": "/api/2.1/jobs/create",
            "jobs_delete": "/api/2.1/jobs/delete",
            "jobs_get": "/api/2.1/jobs/get",
            "jobs_list": "/api/2.1/jobs/list",
            "jobs_reset": "/api/2.1/jobs/reset",
            "jobs_runs_cancel": "/api/2.1/jobs/runs/cancel",
            "jobs_runs_cancel_all": "/api/2.1/jobs/runs/cancel-all",
            "jobs_runs_delete": "/api/2.1/jobs/runs/delete",
            "jobs_runs_export": "/api/2.1/jobs/runs/export",
            "jobs_runs_get": "/api/2.1/jobs/runs/get",
            "jobs_runs_get_output": "/api/2.1/jobs/runs/get-output",
            "jobs_runs_list": "/api/2.1/jobs/runs/list",
            "jobs_runs_repair": "/api/2.1/jobs/runs/repair",
            "jobs_runs_submit": "/api/2.1/jobs/runs/submit",
            "jobs_run_now": "/api/2.1/jobs/run-now",
            "unity_catalog_catalogs_list": "/api/2.1/unity-catalog/catalogs",
            "unity_catalog_schemas_list": "/api/2.1/unity-catalog/schemas",
            "unity_catalog_catalogs_get": "/api/2.1/unity-catalog/catalogs/{catalog_name}",
            "unity_catalog_schemas_get": "/api/2.1/unity-catalog/schemas/{full_name}",
        },
        "preview": {},
    },
    "account": {
        "2.0": {
            "workspaces_create": "/api/2.0/accounts/workspaces",
            "workspaces_delete": "/api/2.0/accounts/workspaces/{workspace_id}",
            "workspaces_get": "/api/2.0/accounts/workspaces/{workspace_id}",
            "workspaces_list": "/api/2.0/accounts/workspaces",
            "workspaces_update": "/api/2.0/accounts/workspaces/{workspace_id}",
            "credentials_list": "/api/2.0/accounts/credentials",
            "credentials_create": "/api/2.0/accounts/credentials",
            "credentials_delete": "/api/2.0/accounts/credentials/{credentials_id}",
            "storage_configurations_list": "/api/2.0/accounts/storage-configurations",
            "storage_configurations_create": "/api/2.0/accounts/storage-configurations",
            "storage_configurations_delete": "/api/2.0/accounts/storage-configurations/{storage_configuration_id}",
            "networks_list": "/api/2.0/accounts/networks",
            "networks_create": "/api/2.0/accounts/networks",
            "networks_delete": "/api/2.0/accounts/networks/{network_id}",
            "private_access_settings_list": "/api/2.0/accounts/private-access-settings",
            "private_access_settings_create": "/api/2.0/accounts/private-access-settings",
            "private_access_settings_delete": "/api/2.0/accounts/private-access-settings/{private_access_settings_id}",
            "vpc_endpoints_list": "/api/2.0/accounts/vpc-endpoints",
            "vpc_endpoints_create": "/api/2.0/accounts/vpc-endpoints",
            "vpc_endpoints_delete": "/api/2.0/accounts/vpc-endpoints/{vpc_endpoint_id}",
            "customer_managed_keys_list": "/api/2.0/accounts/customer-managed-keys",
            "customer_managed_keys_create": "/api/2.0/accounts/customer-managed-keys",
            "customer_managed_keys_delete": "/api/2.0/accounts/customer-managed-keys/{customer_managed_key_id}",
            "users_list": "/api/2.0/accounts/scim/v2/Users",
            "users_get": "/api/2.0/accounts/scim/v2/Users/{user_id}",
            "users_create": "/api/2.0/accounts/scim/v2/Users",
            "users_patch": "/api/2.0/accounts/scim/v2/Users/{user_id}",
            "users_delete": "/api/2.0/accounts/scim/v2/Users/{user_id}",
            "groups_list": "/api/2.0/accounts/scim/v2/Groups",
            "groups_get": "/api/2.0/accounts/scim/v2/Groups/{group_id}",
            "groups_create": "/api/2.0/accounts/scim/v2/Groups",
            "groups_patch": "/api/2.0/accounts/scim/v2/Groups/{group_id}",
            "groups_delete": "/api/2.0/accounts/scim/v2/Groups/{group_id}",
            "budget_policies_list": "/api/2.0/accounts/budget-policies",
            "budget_policies_get": "/api/2.0/accounts/budget-policies/{budget_policy_id}",
            "budget_policies_create": "/api/2.0/accounts/budget-policies",
            "budget_policies_update": "/api/2.0/accounts/budget-policies/{budget_policy_id}",
            "budget_policies_delete": "/api/2.0/accounts/budget-policies/{budget_policy_id}",
            "log_delivery_list": "/api/2.0/accounts/log-delivery",
            "log_delivery_get": "/api/2.0/accounts/log-delivery/{log_delivery_configuration_id}",
            "log_delivery_create": "/api/2.0/accounts/log-delivery",
            "log_delivery_patch": "/api/2.0/accounts/log-delivery/{log_delivery_configuration_id}",
            "log_delivery_delete": "/api/2.0/accounts/log-delivery/{log_delivery_configuration_id}",
        },
        "preview": {},
    },
}
